use rust_qt_binding_generator::*;
use std::{io::prelude::*, path::PathBuf};

fn main()
{
   let (lty, dty, qtpre, cxx) = if cfg!(target_os = "macos") {
      ("framework", "framework", "Qt", "c++")
   } else {
      ("dylib", "native", "Qt5", "stdc++")
   };

   let out_dir = PathBuf::from(std::env::var("OUT_DIR").unwrap());

   let mut config = configuration::parse("bindings.json").unwrap();

   config.overwrite_implementation = true;
   config.cpp_file = out_dir.join("bindings.cc");
   config.rust.dir = out_dir;

   generate_bindings(&config).unwrap();

   let mut config = cmake::Config::new(".");

   if cfg!(debug_assertions) {
      config.cxxflag("-DTYCHO_DEBUG_ASSERTIONS");
   }

   let out_dir = config.build();
   let lib_dir = out_dir.join("lib");
   let etc_dir = out_dir.join("etc");

   println!("cargo:rustc-link-search=native={}", lib_dir.display());
   println!("cargo:rustc-link-lib=static=maraiah-tycho-hermes");

   let fp = std::fs::File::open(etc_dir.join("link.txt")).unwrap();
   let fp = std::io::BufReader::new(fp);

   for ln in fp.lines() {
      let ln = ln.unwrap();

      let path = PathBuf::from(&ln);
      let path = path.parent().unwrap();

      let path = if cfg!(target_os = "macos") {
         path.parent().unwrap()
      } else {
         path
      };

      println!("cargo:rustc-link-search={}={}", dty, path.display());
   }

   println!("cargo:rustc-link-lib={}={}Core", lty, qtpre);
   println!("cargo:rustc-link-lib={}={}Widgets", lty, qtpre);
   println!("cargo:rustc-link-lib={}={}Gui", lty, qtpre);

   println!("cargo:rustc-link-lib=dylib={}", cxx);
}

// EOF
